#include "testlib.h"
#include <map>
#include <vector>
#include <functional>

using namespace std;

using ll = long long;

const int MAXN = 100000;
const int MAXP = 20;

int N;
struct Vertex{
    vector<int> to;
    int p[MAXP];
    int depth;
    int S, E;
}v[MAXN + 1];

int ENTRY = 0;
void PrepareDFS(int it, int p = 0)
{
    v[it].S = ++ENTRY;

    v[it].p[0] = p;
    for (int j = 1; j < MAXP; j++)
    {
        if (v[it].p[j - 1] == 0)
            v[it].p[j] = 0;
        else
            v[it].p[j] = v[v[it].p[j - 1]].p[j - 1];
    }

    v[it].depth = (p == 0) ? 1 : (v[p].depth + 1);

    for (auto t : v[it].to)
    {
        if (t == p)
            continue;
        PrepareDFS(t, it);
    }

    v[it].E = ENTRY;
}

int distance(int a, int b)
{
    if (v[a].depth > v[b].depth)
        swap(a, b);

    if (v[a].S <= v[b].S && v[b].S <= v[a].E)
        return v[b].depth - v[a].depth;

    int it = a;
    for (int i = MAXP - 1; i >= 0; i--)
    {
        if (v[it].p[i] == 0)
            continue;

        int p = v[it].p[i];
        if (v[p].S <= v[b].S && v[b].S <= v[p].E)
        {
            // Do nothing
        }
        else
            it = p;
    }
    int answ = v[it].p[0];

    return v[a].depth + v[b].depth - 2 * v[answ].depth;
}

pair<bool, string> checkPartition(InStream& stream, const ll targetValue)
{
    // Consume permutation, do not validate its correctness yet
    vector<int> input;
    for (int i = 1; i <= N; i++)
    {
        input.push_back(stream.readInt(1, N, "vi"));
    }

    vector<bool> used(N + 1, false);
    ll totalDistance = 0;

    for (int i = 1; i <= N; i++)
    {
        int value = input[i - 1];
        if (value < 1 || N < value)
            return {false, "House number must be in interval [1; N]"};
        if (value == i)
            return {false, "Villager must move between houses"};

        if (used[value])
            return {false, "Given description is not a valid permutation!"};
        used[value] = true;

        totalDistance += distance(i, value);
    }

    if (totalDistance != targetValue)
    {
        return {
            false,
            "Given description doesn't match given total length of paths!"
        };
    }

    return {true, ""};
}

using tbls = tuple<bool, ll, string>;
pair<tbls, tbls> readAns(InStream& stream)
{
    ll smallest = stream.readLong();
    ll largest = stream.readLong();

    bool scorrect, lcorrect;
    string smessage, lmessage;

    tie(scorrect, smessage) = checkPartition(stream, smallest);
    tie(lcorrect, lmessage) = checkPartition(stream, largest);

    return {
        {scorrect, smallest, smessage},
        {lcorrect, largest, lmessage}
    };
}

string check(
        tbls jres,
        tbls pres,
        function<bool(ll, ll)> comp,
        string type,
        int& score
)
{
    bool jok, pok;
    ll jvalue, pvalue;
    string jmessage, pmessage;
    tie(jok, jvalue, jmessage) = jres;
    tie(pok, pvalue, pmessage) = pres;

    if (!jok)
    {
        quitf(_fail, "Jury submission for %s is not formatted correctly: %s", type.c_str(), jmessage.c_str());
    }

    if (!pok)
    {
        return format("incorrect assignment %s", pmessage.c_str());
    }

    // Both jury and participant assignments were valid, compare lengths

    if (comp(jvalue, pvalue))
    {
        return "jury has the better answer";
    }
    else if (jvalue == pvalue)
    {
        score += 1;
        return format("correct answer = %lld", jvalue);
    }
    else
    {
        quitf(_fail,
              ":( participant has the better answer for %s value: jans = %lld, pans = %lld\n",
              type.c_str(),
              jvalue,
              pvalue);
    }
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);

    N = inf.readInt();
    if (N > MAXN) {
        quitf(_fail, "Tree with too many nodes");
    }

    for (int i = 1; i < N; i++) {
        int a = inf.readInt();
        int b = inf.readInt();
        v[a].to.push_back(b);
        v[b].to.push_back(a);
    }

    PrepareDFS(1);

    tbls jsmallest, jlargest, psmallest, plargest;
    tie(jsmallest, jlargest) = readAns(ans);
    tie(psmallest, plargest) = readAns(ouf);

    int score = 0;

    string smallest = check(jsmallest, psmallest, std::less<ll>(), "smallest", score);
    string largest = check(jlargest, plargest, std::greater<ll>(), "largest", score);

    if (score == 0)
        quitf(_wa, "%s; %s;\n", smallest.c_str(), largest.c_str());
    else
        quitp(score / 2.0, "%s; %s;\n", smallest.c_str(), largest.c_str());
}
